@@ -1,3 +1,16 @@
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
+# implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 """ Module defining the Django auth backend class for the Keystone API. """
 
 import logging
@@ -44,7 +57,8 @@ class KeystoneBackend(object):
         Note: this required monkey-patching the ``contrib.auth`` middleware
         to make the ``request`` object available to the auth backend class.
         """
-        if user_id == self.request.session["user_id"]:
+        if (hasattr(self, 'request') and
+                user_id == self.request.session["user_id"]):
             token = self.request.session['token']
             endpoint = self.request.session['region_endpoint']
             services_region = self.request.session['services_region']
@@ -60,6 +74,9 @@ class KeystoneBackend(object):
         LOG.debug('Beginning user authentication for user "%s".' % username)
 
         insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
+        ca_cert = getattr(settings, "OPENSTACK_SSL_CACERT", None)
+        endpoint_type = getattr(
+            settings, 'OPENSTACK_ENDPOINT_TYPE', 'publicURL')
 
         keystone_client = get_keystone_client()
         try:
@@ -69,6 +86,7 @@ class KeystoneBackend(object):
                 password=password,
                 auth_url=auth_url,
                 insecure=insecure,
+                cacert=ca_cert,
                 debug=settings.DEBUG)
 
             unscoped_auth_ref = client.auth_ref
@@ -119,6 +137,7 @@ class KeystoneBackend(object):
                         token=unscoped_auth_ref.auth_token,
                         auth_url=auth_url,
                         insecure=insecure,
+                        cacert=ca_cert,
                         debug=settings.DEBUG)
                     auth_ref = client.auth_ref
                     break
@@ -134,9 +153,10 @@ class KeystoneBackend(object):
         self.check_auth_expiry(auth_ref)
 
         # If we made it here we succeeded. Create our User!
-        user = create_user_from_token(request,
-                                      Token(auth_ref),
-                                      client.service_catalog.url_for())
+        user = create_user_from_token(
+            request,
+            Token(auth_ref),
+            client.service_catalog.url_for(endpoint_type=endpoint_type))
 
         if request is not None:
             request.session['unscoped_token'] = unscoped_token.id
