@@ -1,3 +1,16 @@
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
+# implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import mox
 
 from django import test
@@ -11,6 +24,7 @@ from keystoneclient.v3 import client as client_v3
 
 from .data_v2 import generate_test_data as data_v2
 from .data_v3 import generate_test_data as data_v3
+import copy
 
 
 DEFAULT_DOMAIN = settings.OPENSTACK_KEYSTONE_DEFAULT_DOMAIN
@@ -52,12 +66,14 @@ class OpenStackAuthTestsV2(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False)\
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.tenants.list().AndReturn(tenants)
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.tenant_two.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndReturn(self.keystone_client_scoped)
@@ -94,6 +110,7 @@ class OpenStackAuthTestsV2(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False)\
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.tenants.list().AndReturn(tenants)
@@ -101,12 +118,14 @@ class OpenStackAuthTestsV2(test.TestCase):
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.tenant_two.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndRaise(exc)
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.tenant_one.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndReturn(self.keystone_client_scoped)
@@ -141,6 +160,7 @@ class OpenStackAuthTestsV2(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False)\
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.tenants.list().AndReturn(tenants)
@@ -148,12 +168,14 @@ class OpenStackAuthTestsV2(test.TestCase):
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.tenant_two.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndRaise(exc)
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.tenant_one.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndRaise(exc)
@@ -189,6 +211,7 @@ class OpenStackAuthTestsV2(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False)\
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.tenants.list().AndReturn([])
@@ -223,6 +246,7 @@ class OpenStackAuthTestsV2(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False).AndRaise(exc)
 
         self.mox.ReplayAll()
@@ -254,6 +278,7 @@ class OpenStackAuthTestsV2(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False).AndRaise(exc)
 
         self.mox.ReplayAll()
@@ -279,6 +304,7 @@ class OpenStackAuthTestsV2(test.TestCase):
         unscoped = self.data.unscoped_access_info
         scoped = self.data.scoped_access_info
         sc = self.data.service_catalog
+        et = getattr(settings, 'OPENSTACK_ENDPOINT_TYPE', 'publicURL')
 
         form_data = {'region': settings.OPENSTACK_KEYSTONE_URL,
                      'domain': DEFAULT_DOMAIN,
@@ -293,20 +319,23 @@ class OpenStackAuthTestsV2(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False) \
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.tenants.list().AndReturn(tenants)
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.tenant_two.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndReturn(self.keystone_client_scoped)
 
-        self.ks_client_module.Client(auth_url=sc.url_for(),
+        self.ks_client_module.Client(auth_url=sc.url_for(endpoint_type=et),
                                      tenant_id=tenant.id,
                                      token=scoped.auth_token,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False) \
                 .AndReturn(self.keystone_client_scoped)
 
@@ -360,12 +389,14 @@ class OpenStackAuthTestsV2(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False) \
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.tenants.list().AndReturn(tenants)
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.tenant_two.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndReturn(self.keystone_client_scoped)
@@ -405,6 +436,48 @@ class OpenStackAuthTestsV2(test.TestCase):
         self.test_switch_region(next='/next_url')
 
 
+def EndpointMetaFactory(endpoint_type):
+    def endpoint_wrapper(func):
+        def new_func(*args, **kwargs):
+            _endpoint_type = getattr(settings, 'OPENSTACK_ENDPOINT_TYPE', None)
+            # set settings.OPENSTACK_ENDPOINT_TYPE to given value
+            setattr(settings, 'OPENSTACK_ENDPOINT_TYPE', endpoint_type)
+            # ensure that ret won't be touched by del/setattr below
+            ret = copy.deepcopy(func(*args, **kwargs))
+            # and restore it
+            if _endpoint_type is None:
+                del settings.OPENSTACK_ENDPOINT_TYPE
+            else:
+                setattr(settings, 'OPENSTACK_ENDPOINT_TYPE', _endpoint_type)
+            return ret
+        return new_func
+
+    class EndPointMeta(type):
+        # wrap each test with OPENSTACK_ENDPOINT_TYPE parameter set/restore
+        def __new__(cls, name, bases, attrs):
+            base, = bases
+            for k, v in base.__dict__.iteritems():
+                if not k.startswith('__') and getattr(v, '__call__', None):
+                    attrs[k] = endpoint_wrapper(v)
+            return super(EndPointMeta, cls).__new__(cls, name, bases, attrs)
+    return EndPointMeta
+
+
+class OpenStackAuthTestsV2WithPublicURL(OpenStackAuthTestsV2):
+    """Test V2 with settings.OPENSTACK_ENDPOINT_TYPE = 'publicURL'."""
+    __metaclass__ = EndpointMetaFactory('publicURL')
+
+
+class OpenStackAuthTestsV2WithInternalURL(OpenStackAuthTestsV2):
+    """Test V2 with settings.OPENSTACK_ENDPOINT_TYPE = 'internalURL'."""
+    __metaclass__ = EndpointMetaFactory('internalURL')
+
+
+class OpenStackAuthTestsV2WithAdminURL(OpenStackAuthTestsV2):
+    """Test V2 with settings.OPENSTACK_ENDPOINT_TYPE = 'adminURL'."""
+    __metaclass__ = EndpointMetaFactory('adminURL')
+
+
 class OpenStackAuthTestsV3(test.TestCase):
     def setUp(self):
         super(OpenStackAuthTestsV3, self).setUp()
@@ -444,6 +517,7 @@ class OpenStackAuthTestsV3(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False)\
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.projects.list(user=user.id) \
@@ -451,6 +525,7 @@ class OpenStackAuthTestsV3(test.TestCase):
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.project_two.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndReturn(self.keystone_client_scoped)
@@ -486,6 +561,7 @@ class OpenStackAuthTestsV3(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False)\
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.projects.list(user=user.id) \
@@ -494,12 +570,14 @@ class OpenStackAuthTestsV3(test.TestCase):
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.project_two.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndRaise(exc)
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.project_one.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndReturn(self.keystone_client_scoped)
@@ -535,6 +613,7 @@ class OpenStackAuthTestsV3(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False)\
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.projects.list(user=user.id) \
@@ -543,12 +622,14 @@ class OpenStackAuthTestsV3(test.TestCase):
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.project_two.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndRaise(exc)
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.project_one.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndRaise(exc)
@@ -585,6 +666,7 @@ class OpenStackAuthTestsV3(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False)\
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.projects.list(user=user.id) \
@@ -620,6 +702,7 @@ class OpenStackAuthTestsV3(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False).AndRaise(exc)
 
         self.mox.ReplayAll()
@@ -651,6 +734,7 @@ class OpenStackAuthTestsV3(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False).AndRaise(exc)
 
         self.mox.ReplayAll()
@@ -676,6 +760,7 @@ class OpenStackAuthTestsV3(test.TestCase):
         unscoped = self.data.unscoped_access_info
         scoped = self.data.scoped_access_info
         sc = self.data.service_catalog
+        et = getattr(settings, 'OPENSTACK_ENDPOINT_TYPE', 'publicURL')
 
         form_data = {'region': settings.OPENSTACK_KEYSTONE_URL,
                      'domain': DEFAULT_DOMAIN,
@@ -691,6 +776,7 @@ class OpenStackAuthTestsV3(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False) \
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.projects.list(user=user.id) \
@@ -698,13 +784,15 @@ class OpenStackAuthTestsV3(test.TestCase):
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.project_two.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndReturn(self.keystone_client_scoped)
-        self.ks_client_module.Client(auth_url=sc.url_for(),
+        self.ks_client_module.Client(auth_url=sc.url_for(endpoint_type=et),
                                      tenant_id=project.id,
                                      token=scoped.auth_token,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False) \
                 .AndReturn(self.keystone_client_scoped)
 
@@ -759,6 +847,7 @@ class OpenStackAuthTestsV3(test.TestCase):
                                      username=user.name,
                                      user_domain_name=DEFAULT_DOMAIN,
                                      insecure=False,
+                                     cacert=None,
                                      debug=False) \
                 .AndReturn(self.keystone_client_unscoped)
         self.keystone_client_unscoped.projects.list(user=user.id) \
@@ -766,6 +855,7 @@ class OpenStackAuthTestsV3(test.TestCase):
         self.ks_client_module.Client(auth_url=settings.OPENSTACK_KEYSTONE_URL,
                                      tenant_id=self.data.project_two.id,
                                      insecure=False,
+                                     cacert=None,
                                      token=unscoped.auth_token,
                                      debug=False) \
                 .AndReturn(self.keystone_client_scoped)
@@ -803,3 +893,18 @@ class OpenStackAuthTestsV3(test.TestCase):
 
     def test_switch_region_with_next(self, next=None):
         self.test_switch_region(next='/next_url')
+
+
+class OpenStackAuthTestsV3WithPublicURL(OpenStackAuthTestsV3):
+    """Test V3 with settings.OPENSTACK_ENDPOINT_TYPE = 'publicURL'."""
+    __metaclass__ = EndpointMetaFactory('publicURL')
+
+
+class OpenStackAuthTestsV3WithInternalURL(OpenStackAuthTestsV3):
+    """Test V3 with settings.OPENSTACK_ENDPOINT_TYPE = 'internalURL'."""
+    __metaclass__ = EndpointMetaFactory('internalURL')
+
+
+class OpenStackAuthTestsV3WithAdminURL(OpenStackAuthTestsV3):
+    """Test V3 with settings.OPENSTACK_ENDPOINT_TYPE = 'adminURL'."""
+    __metaclass__ = EndpointMetaFactory('adminURL')
