@@ -1,3 +1,16 @@
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
+# implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import hashlib
 import logging
 
@@ -17,13 +30,12 @@ LOG = logging.getLogger(__name__)
 
 def set_session_from_user(request, user):
     request.session['token'] = user.token
-    if 'token_list' not in request.session:
-        request.session['token_list'] = []
-    token_tuple = (user.endpoint, user.token.id)
-    request.session['token_list'].append(token_tuple)
     request.session['user_id'] = user.id
     request.session['region_endpoint'] = user.endpoint
     request.session['services_region'] = user.services_region
+    # Update the user object cached in the request
+    request._cached_user = user
+    request.user = user
 
 
 def create_user_from_token(request, token, endpoint, services_region=None):
@@ -77,9 +89,9 @@ class Token(object):
         self.domain = domain
 
         if auth_ref.version == 'v2.0':
-            self.roles = auth_ref['user']['roles']
+            self.roles = auth_ref['user'].get('roles', [])
         else:
-            self.roles = auth_ref['roles']
+            self.roles = auth_ref.get('roles', [])
 
         if get_keystone_version() < 3:
             self.serviceCatalog = auth_ref.get('serviceCatalog', [])
@@ -210,6 +222,7 @@ class User(AnonymousUser):
     def authorized_tenants(self):
         """ Returns a memoized list of tenants this user may access. """
         insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
+        ca_cert = getattr(settings, "OPENSTACK_SSL_CACERT", None)
 
         if self.is_authenticated() and self._authorized_tenants is None:
             endpoint = self.endpoint
@@ -220,6 +233,7 @@ class User(AnonymousUser):
                     auth_url=endpoint,
                     token=token.id,
                     insecure=insecure,
+                    cacert=ca_cert,
                     debug=settings.DEBUG)
             except (keystone_exceptions.ClientException,
                     keystone_exceptions.AuthorizationFailure):
@@ -262,7 +276,7 @@ class User(AnonymousUser):
                 if service['type'] == 'identity':
                     continue
                 for endpoint in service['endpoints']:
-                    if not endpoint['region'] in regions:
+                    if endpoint['region'] not in regions:
                         regions.append(endpoint['region'])
         return regions
 
