@@ -1,7 +1,19 @@
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
+# implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 import logging
 
-from threading import Thread
-
+import django
 from django import shortcuts
 from django.conf import settings
 from django.contrib.auth import REDIRECT_FIELD_NAME
@@ -44,7 +56,13 @@ def login(request):
         initial.update({'region': requested_region})
 
     if request.method == "POST":
-        form = curry(Login, request)
+        # NOTE(saschpe): Since https://code.djangoproject.com/ticket/15198,
+        # the 'request' object is passed directly to AuthenticationForm in
+        # django.contrib.auth.views#login:
+        if django.VERSION >= (1, 6):
+            form = curry(Login)
+        else:
+            form = curry(Login, request)
     else:
         form = curry(Login, initial=initial)
 
@@ -76,32 +94,36 @@ def logout(request):
     msg = 'Logging out user "%(username)s".' % \
         {'username': request.user.username}
     LOG.info(msg)
-    if 'token_list' in request.session:
-        t = Thread(target=delete_all_tokens,
-                   args=(list(request.session['token_list']),))
-        t.start()
+    endpoint = request.session.get('region_endpoint')
+    token = request.session.get('token')
+    if token and endpoint:
+        delete_token(endpoint=endpoint, token_id=token.id)
     """ Securely logs a user out. """
     return django_logout(request)
 
 
-def delete_all_tokens(token_list):
+def delete_token(endpoint, token_id):
+    """Delete a token."""
+
     insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
-    for token_tuple in token_list:
-        try:
-            endpoint = token_tuple[0]
-            token = token_tuple[1]
-            if get_keystone_version() < 3:
-                client = keystone_client_v2.Client(endpoint=endpoint,
-                                                token=token,
-                                                insecure=insecure,
-                                                debug=settings.DEBUG)
-                client.tokens.delete(token=token)
-            else:
-                # FIXME: KS-client does not have delete token available
-                # Need to add this later when it is exposed.
-                pass
-        except keystone_exceptions.ClientException as e:
-            LOG.info('Could not delete token')
+    ca_cert = getattr(settings, "OPENSTACK_SSL_CACERT", None)
+    try:
+        if get_keystone_version() < 3:
+            client = keystone_client_v2.Client(
+                endpoint=endpoint,
+                token=token_id,
+                insecure=insecure,
+                cacert=ca_cert,
+                debug=settings.DEBUG
+            )
+            client.tokens.delete(token=token_id)
+            LOG.info('Deleted token %s' % token_id)
+        else:
+            # FIXME: KS-client does not have delete token available
+            # Need to add this later when it is exposed.
+            pass
+    except keystone_exceptions.ClientException as e:
+        LOG.info('Could not delete token')
 
 
 @login_required
@@ -110,15 +132,16 @@ def switch(request, tenant_id, redirect_field_name=REDIRECT_FIELD_NAME):
     LOG.debug('Switching to tenant %s for user "%s".'
               % (tenant_id, request.user.username))
     insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
+    ca_cert = getattr(settings, "OPENSTACK_SSL_CACERT", None)
     endpoint = request.user.endpoint
     try:
         if get_keystone_version() >= 3:
             endpoint = endpoint.replace('v2.0', 'v3')
-
         client = get_keystone_client().Client(tenant_id=tenant_id,
                                               token=request.user.token.id,
                                               auth_url=endpoint,
                                               insecure=insecure,
+                                              cacert=ca_cert,
                                               debug=settings.DEBUG)
         auth_ref = client.auth_ref
         msg = 'Project switch successful for user "%(username)s".' % \
@@ -138,11 +161,16 @@ def switch(request, tenant_id, redirect_field_name=REDIRECT_FIELD_NAME):
         redirect_to = settings.LOGIN_REDIRECT_URL
 
     if auth_ref:
+        old_endpoint = request.session.get('region_endpoint')
+        old_token = request.session.get('token')
+        if old_token and old_endpoint and old_token.id != auth_ref.auth_token:
+            delete_token(endpoint=old_endpoint, token_id=old_token.id)
         user = create_user_from_token(request, Token(auth_ref), endpoint)
         set_session_from_user(request, user)
     return shortcuts.redirect(redirect_to)
 
 
+@login_required
 def switch_region(request, region_name,
                   redirect_field_name=REDIRECT_FIELD_NAME):
     """
